% This function is part of the reproducible research addendum of the research
% paper:
% Peter Balazs, Nicki Holighaus, Thibaud Necciari, and Diana Stoeva
% "Frame Theory for Signal Processing in Psychoacoustics"
%
% This file along with the auxiliary files female.wav and alldiv.m reproduce Fig.10.
%
% To run this file, you will need the LTFAT Toolbox V.2.1.1 or later,
% available at: ltfat.sourceforge.net
% 
% Author: T. Necciari, 26.09.2016

function Fig10_audlet_gfb_comparison(red_fac,V)

if nargin < 2
   V = 6;% Default filters' spacing
   if nargin < 1
    red_fac = 2; % Default redundancy factor
   end
end

%% Load test signal and set up the filterbank parameters
[s,sr]=wavload('female.wav');
L = length(s);
Ls = ceil(L/sr)*sr;
s = [s;zeros(Ls-L,1)];
fmin = 0;
fmax = sr/2;

%----------------------AUDlet filters--------------------------------------
% Here we make a probe to obtain the center frequencies and the fractional
% sub-sampling factors. The latter will be rounded to integer sub-sampling
% in the following...
spac = 1/V;
[erblets,a,fc,Lerbfb] = audfilters(sr,Ls,'fractional','spacing',spac,'flow',fmin,'fhigh',fmax);

% Compute corrected downsampling factors: The gammatone FIR filters do not 
% work with fractional subsampling. Therefore, we compute the next possible 
% integer downsampling factors. 
DIVS = [1,alldiv(Ls)];
a_old = a(:,1)./a(:,2);
a = a(:,1)./a(:,2)/red_fac;
for kk=1:length(a)
  ind = find(DIVS <= a(kk),1,'last');
  a(kk) = DIVS(ind);
end 
a_ratio = sqrt(a./a_old); 

disp('------ Non-uniform ERBlet filter bank ------')
% Adjust the ERBlet normalization to the modified downsampling factors
erblets = filterbankscale(erblets,a_ratio');

% Compute the filterbank analysis
c_erb = filterbank(s,erblets,a);

% Synthesis
if red_fac >= 1 % Painless case, compute the dual filters directly 
    erbletsd = filterbankrealdual(erblets,a,Lerbfb);
    s_r1 = 2*real(ifilterbank(c_erb,erbletsd,a));
else % Non-painless, apply the canonical dual frame iteratively using CG
    F  = frame('filterbankreal',erblets,a,length(erblets));
    c_F=framenative2coef(F,c_erb);
    [s_r1,~,~] = frsyniter(F,c_F,Ls,'tol',10^-14);
end
err_erblet = norm(s-s_r1)/norm(s);
fprintf('Relative error of reconstruction: %e \n\n',err_erblet);

%% Gammatone filter bank
disp('------ Non-uniform gammatone filter bank ------')

% Initialize parameters: Try to get as close as possible to the AUDlet
Lgfb = 12000;% We specify 12000 samples because the gammatonefir function actually uses only the half of it...

% Compute the normalization factors for the non-uniform GFB
scal=sqrt(a);

% Compute and normalize filters
gfb = gammatonefir(fc,sr,Lgfb,1);
gfb = filterbankscale(gfb,scal);

% In order to make the Gammatone filters close to tight (with frame bound 1) 
% in the main frequency range, we normalize the filters again with an estimate 
% of the average of the filterbank frequency response.
gfbresp = filterbankresponse(gfb,a,Ls);
ind = round(length(gfbresp)/4);
normfac = sqrt((min(gfbresp(ind-500:ind+500))+max(gfbresp(ind-500:ind+500)))/2);

% These will be the final Gammatone filters
gfb = cellfun(@(gEl) setfield(gEl,'h',gEl.h/normfac),gfb,'UniformOutput',0);
% Finally adjust first and last filters
gfb{1}.h = gfb{1}.h./sqrt(2);
gfb{end}.h = gfb{end}.h./sqrt(2);

% Compute the filterbank analysis
c_gfb = filterbank(s,gfb,a);

% Synthesis
s_r3 = 2*real(ifilterbank(c_gfb,gfb,a,Ls));% Synthesis using time-reversed versions of the analysis filters
err_gfb = norm(s-s_r3)/norm(s);
fprintf('Relative error of reconstruction: %e \n\n',err_gfb); 

%% Do figures...
%     Plot ERB coefficients
    figure('Name','Hann AUDlet coefficients')
    plotfilterbank(c_erb,a,fc,sr,60,'audtick');
    xlim([0 2.5])
    colormap(flipud(gray));
    
%     Plot GFB coefficients
    figure('Name','Gammatone coefficients');
    plotfilterbank(c_gfb,a,fc,sr,60,'audtick');
    xlim([0 2.5])
    colormap(flipud(gray));
end

% eof