% This function is part of the reproducible research addendum of the research
% paper:
% Peter Balazs, Nicki Holighaus, Thibaud Necciari, and Diana Stoeva
% "Frame Theory for Signal Processing in Psychoacoustics"
%
% This file along with the auxiliary file Manowar.wav reproduce Fig.11.
% 
% For details on the irrelevance filter algorithm refer to:
% 
% Balazs, P.; Laback, B.; Eckel, G. & Deutsch, W. A.
% IEEE Trans. Audio, Speech, Language Process., 2010, vol. 18, pp. 34-49
%
% To run this file, you will need the LTFAT Toolbox V.2.1.1 or later,
% available at: ltfat.sourceforge.net
% 
% Author: T. Necciari and P. Balazs, 26.09.2016

% Set transform parameters
a=32;% hop size
M=256;% Number of frequency channels
Nwin=256;% Window size

winname=firwin('hamming',Nwin,'1');

% Set parameters for irrelevance filter
offset=-259e-2; % offset parameter of the unprocessed signal 
NBark=512; % frequency bins in the bark scale

[f,fs]=wavload('Manowar.wav');
ratio = fs/16000;
f = resample(f,16000,fs);
fs = fs/ratio;

% SPL normalization...
f = gaindb(f,100);

% Compute discrete Gabor transform 
[c,Ls]=dgtreal(f,winname,a,M);
L=size(c,2)*a;
[lower,upper]=gabframebounds(winname,a,M,L);

figure
plotdgtreal(c,a,M,fs,'dynrange',90); %spectogram
title('Original DGT');

% Separation of amplitude and phase 
s=abs(c).^2; % extract the amplitude
theta=angle(c); %extract the phase

% Convert to Bark scale
freq=linspace(0,fs/2,floor(M/2)+1); % frequency samples in Hz
y = audspace(0,fs/2,NBark,'bark'); 
auddd = freqtoaud(y,'bark');
S=zeros(NBark,size(c,2));
indx_Hz_Bark = zeros(size(freq));
for ii=1:size(c,1)
    [val,ind]=min(abs(freq(ii)-y));
    indx_Hz_Bark(ii) = ind; 
end
S(indx_Hz_Bark,:)=s;

% Compute the spreading function
l=27; % lower frequency slope
u=24; % upper frequency slope
e=3e-1; % non-negative number, helps to control the smoothness of the function around 0
w=max(auddd)*linspace(-1,1,NBark); % spread value

% Shift to (0,0)
xmax=sqrt(e/(l*u))*(l-u)/2; %maximum of shape function on x-label
ymax=-sqrt(e*u*l); %maximum of shape function on y-label

F=(l-u).*(xmax+w)./2-((l+u)/2).*sqrt(e+(xmax+w).^2); %shape function
B=F-ymax; %Basilar membrane spreading function shifted to (0,0)

B(NBark/2-1:NBark/2+1)=-Inf; % on the linear scale
% Because the parameter epsilon is set to the resolution of the frequency
% analysis in Hz.

Bpow=10.^(B./10);

% Convolution in the Bark domain
H=zeros(size(S));
% convolution as an element-wise multiplication
for kk=1:size(c,2)
    H(:,kk)= conv(S(:,kk),Bpow,'same');
end

% Transform from Bark back to Hz 
conv_fun=H(indx_Hz_Bark,:);

% Weight the threshold function at the corresponding bins by the critical
% bandwidth
CB =  audfiltbw(freq,'bark');
W_fun = zeros(size(conv_fun));
 for ll=1:length(CB)
     W_fun(ll,:)=conv_fun(ll,:).*(100/CB(ll));
 end

% Apply the offset 
I=W_fun./(10^(offset/10));% I is the final irrelevance threshold! 

% Gabor Filter by Thresholding 
m = s>I;

% Compare s and I at the time step where s has a peak of energy
[~,ind] = max(max(s,[],1));
figure
plot(freq*1e-3,10*log10(s(:,ind)),'k--','LineWidth',1), hold on
plot(freq*1e-3,10*log10(I(:,ind)),'k-','LineWidth',2), hold off
xlabel('Frequency (kHz)')
ylabel('Amplitude (dB)')
legend('Original spectrum','Irrelevance threshold')
title('Irrelevance threshold')

% Multiplication with the Gabor coefficients/apply the filter to the signal
c_mask=m.*c; 

% Display the mask
figure
plotdgtreal(m,a,M,fs,'dynrange',90,'nocolorbar');
colormap(gray)

% And the "maskex" spectrogram
figure
plotdgtreal(c_mask,a,M,fs,'dynrange',90);
title('Masked DGT')

% Compute the inverse DGT
r=idgtreal(c_mask,winname/upper,a,M,Ls);

% Undo SPL normalization
r = gaindb(r,-100);
f = gaindb(f,-100);

disp('Relative reconstruction error:')
norm(f-r)/norm(f) 

fprintf('Percent of irrelevant components:')
(numel(c)-nnz(m))/numel(c)*100

% eof