% This script is part of the reproducible research addendum of the research
% paper:
% Peter Balazs, Nicki Holighaus, Thibaud Necciari, and Diana Stoeva
% "Frame Theory for Signal Processing in Psychoacoustics"
%
% This file along with the auxiliary files jumpshortorig.wav and 
% jumpshortorig_mask_bw.png reproduce Fig.7 in the paper.
% 
% To run this file, you will need the LTFAT Toolbox V.2.1.1 or later,
% available at: ltfat.sourceforge.net
% 
% Author:
% Peter Balazs, last modification 26.09.2016
%
% This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
% To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/

function Fig7_Ex_Multiplier(filename)

%% loading the signal
if nargin < 1
    filename = 'jumpshortorig';
end
[sig,fs,~,FIDX] = wavload([filename '.WAV']);

% structure for signal
signal.fs = fs;              %sampling rate
signal.nBits = FIDX(7);        % bit rate
signal.nCh = FIDX(5); % number of channels
signal.name = filename;      %filename

[Ls, mmm] = size(sig);
if mmm ~= signal.nCh
    disp('Iternal error: number of channels');
    clear signal;
    return
end
signal.sig = sig;             % samples
signal.len = Ls;             % length of signal
signal.unit.Temp = 'Samples';
signal.unit.Amp  = 'linear'; % linear from -1 to 1

%% parameters for Gabor transform and plot
Nwin_ms = 80;                    % length of window in ms
Nwin = round(Nwin_ms*fs/1000);   % length of window
winname = 'hanning';             % window type

overlap = 8;                %overlap, redundancy in time
a = round(Nwin/overlap);         % time shift, hop size
zeropad = 1;           % use zeropadding? redundancy in frequency.
M = Nwin*(zeropad);            % number of channels, length of FFT (do it painless)

% plot parameters
maxfreq = 6000; % maximal displayed frequency
dynrange = 50; % dynamical range
xcm = nicecolormap; % choose the nice colormap

%% frame creation
% using the frame concept in LTFAT allows an easy extension later.
% e.g. to wavelets, NSGT or general frames
[Fa,Fs]=framepair('dgtreal',{winname,Nwin},'dual',a,M); % create the frames

Coeff_Orig = frana(Fa,signal.sig); % frame analysis,
% for the discrete Gabor transform equivalent to dgtreal(signal.sig,g,a,M); 

%% plot the original signal                                    

figure(1);
subplot(2,2,1)

plotframe(Fa,Coeff_Orig,signal.fs,'dynrange',dynrange);
set(gca,'YLim',[0 maxfreq]);
CCC = get(gca,'CLim');
title(sprintf('TF representation of original signal'));
colormap(xcm);

%% mask/symbol for the multiplier   
%% reads the symbol from outside 
mmss_dB = -40; % minimal number in dB
mmss = 10^(mmss_dB/20);
% read symbol from file
% save it to the right filename !!
mask_orig=double(imread(sprintf('%s_mask_bw.png',filename)));

mask = mask_orig;

% raise to the mimimum
iii = mask<mmss;
mask(iii) = mmss;
mask = 1-mask;

mask = mask(:); % for the frame framework a vector instead of a matrix is needed.

% plot the mask
subplot('Position', [.5703 .5843 .2461 .341]);
        plotframe(Fa,mask,signal.fs);         
    set(gca,'YLim',[0 maxfreq]);
colorbar('off');
title('Gabor mask: symbol of multiplier');

%% TF plot of the result of the multiplier 
subplot('Position', [.35, 0.1, .35, .35]);
filtered_signal = framemul(signal.sig,Fa,Fs,mask);

Coeff_Filt = frana(Fa,filtered_signal);
plotframe(Fa,Coeff_Filt,signal.fs,'dynrange',dynrange);
set(gca,'YLim',[0 maxfreq])
set(gca,'CLim',CCC);
title('TF representation of modified signal');

return


function xcm = nicecolormap()
xcm = [         0         0    0.2000
         0         0    0.2076
         0         0    0.2152
         0         0    0.2229
         0         0    0.2305
         0         0    0.2381
         0         0    0.2457
         0         0    0.2533
         0         0    0.3778
         0         0    0.5022
         0         0    0.6267
         0         0    0.7511
         0         0    0.8756
         0         0    1.0000
         0    0.0857    0.9143
         0    0.1714    0.8286
         0    0.2571    0.7429
         0    0.3429    0.6571
         0    0.4286    0.5714
         0    0.5143    0.4857
         0    0.6000    0.4000
    0.0909    0.6364    0.3636
    0.1818    0.6727    0.3273
    0.2727    0.7091    0.2909
    0.3636    0.7455    0.2545
    0.4545    0.7818    0.2182
    0.5455    0.8182    0.1818
    0.6364    0.8545    0.1455
    0.7273    0.8909    0.1091
    0.8182    0.9273    0.0727
    0.9091    0.9636    0.0364
    1.0000    1.0000         0
    1.0000    0.8750         0
    1.0000    0.7500         0
    1.0000    0.6250         0
    1.0000    0.5000         0
    1.0000    0.3750         0
    1.0000    0.2500         0
    1.0000    0.1250         0
    1.0000         0         0
    1.0000    0.0196    0.0196
    1.0000    0.0392    0.0392
    1.0000    0.0588    0.0588
    1.0000    0.0784    0.0784
    1.0000    0.1936    0.1936
    1.0000    0.3088    0.3088
    1.0000    0.4240    0.4240
    1.0000    0.5392    0.5392
    1.0000    0.6544    0.6544
    1.0000    0.7696    0.7696
    1.0000    0.8848    0.8848
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000
    1.0000    1.0000    1.0000];

