% This script is part of the reproducible research addendum of the research
% paper:
% Peter Balazs, Nicki Holighaus, Thibaud Necciari, and Diana Stoeva
% "Frame Theory for Signal Processing in Psychoacoustics"
%
% This file is an example script to show how to perform analysis and 
% synthesis using the AUDlet filter bank. Various FB settings are proposed.
%
% To run this file, you will need the LTFAT Toolbox V.2.1.1 or later,
% available at: ltfat.sourceforge.net
% 
% Run without input parameters, a medium redundancy transform using 1 filter/ERB
% will be computed. You can adjust the redundancy and the filters' spacing by 
% specifying the input parameters 'red_fac' and 'V'.
% 
% The third input parameter 'PLOT' can be set to 0 to suppress plotting of
% the results.
% 
% Author: T. Necciari, 26.09.2016

function audlet_fb_demo(red_fac,V,PLOT)

if nargin < 3
    PLOT = 1; % Default: plot everything
    if nargin < 2
        V = 1;% Default filters' spacing
        if nargin < 1
            red_fac = 1; % Default redundancy factor
        end
    end
end

%% Load test signal and set up the filterbank parameters
% Load a WAV file
[s,sr]=wavload('female.wav');
Ls = length(s);

% One of the LTFAT test speech signals...
% [s,sr]=greasy;
% Ls = length(s);

% Random signal
% Ls = 263614;
% s = rand(Ls,1)-.5;
% sr = 44100;

spac = 1/V;

%% 1st example: Construct a non-uniform FB on the ERB scale with fractional sub-sampling and a Hann window
disp('------ Example 1: Non-uniform ERBlet FB with fractional sub-sampling and Hann windows ------')
[erblets1,a1,fc1,L1] = audfilters(sr,Ls,'erb','fractional','spacing',spac,'redmul',red_fac);
% To obtain a non-uniform FB with integer sub-sampling, use the flag 'regsampling' instead of 'fractional'.
% See the help of AUDFILTERS for more options.
% Alternatively, one can use the equivalent function call:
% [erblets1,a1,fc1,Lerbfb1] = erbfilters(sr,Ls,'fractional','spacing',spac,'bwmul',spac,'redmul',red_fac);
% The function ERBFILTERS is a wrapper of AUDFILTERS for the ERB scale.

s1 = postpad(s,L1);

% Compute and display FB properties and redundancy
K1 = length(a1)-1;% The zero-frequency filter is included in K!
fprintf('The filter bank features K = %i filters from 0 to %i Hz + one filter at the Nyquist frequency\n',K1,round(fc1(end-1)));
fprintf('Filters'' spacing V = %i filter(s)/ERB\n',V);
fprintf('Redundancy factor = %g\n',red_fac);
R1 = (sum(2*a1(:,2))-sum(a1([1,end],2)))/L1;
fprintf('Actual redundancy of the system: %g \n', R1);

% Analysis
c_erb1 = filterbank(s1,erblets1,a1);

% Synthesis
if R1 >= 1
    if red_fac >= 1 % Painless case, compute the dual filters directly 
        erbletsd1 = filterbankrealdual(erblets1,a1,L1);
        s_r1 = 2*real(ifilterbank(c_erb1,erbletsd1,a1));
    else % Non-painless, apply the canonical dual frame iteratively using CG
        disp('An iterative synthesis using CG is performed...')
        F  = frame('filterbankreal',erblets1,a1,length(erblets1));
        c_F=framenative2coef(F,c_erb1);
        [s_r1,~,~] = frsyniter(F,c_F,L1,'tol',10^-14);
    end
    err1 = norm(s1-s_r1)/norm(s1);
    fprintf('Relative error of reconstruction: %e \n\n',err1);
else 
    disp('Redundancy < 1, no synthesis is performed')
end

% Do figures...
if PLOT
%     Plot frequency responses of individual filters
    if red_fac >= 1
        figure('Name','Example 1: Hann Filter responses');
        subplot(2,1,1);
        filterbankfreqz(erblets1,a1,L1,sr,'plot','linabs','posfreq');
        title('Analysis filters')
        subplot(2,1,2);
        filterbankfreqz(erbletsd1,a1,L1,sr,'plot','linabs','posfreq');
        title('Dual filters')
    end
%     Plot FB response
    figure('Name','Example 1: FB response')
    filterbankresponse(erblets1,a1,L1,sr,'real','plot');
%     Plot ERB coefficients
    figure('Name','Example 1: Analysis coefficients')
    plotfilterbank(c_erb1,a1,fc1,sr,100,'audtick');
    title('Non-uniform ERBlet FB using Hann windows')
end

%% 2nd example: Construct a uniform FB on the Bark scale with fractional sub-sampling and a cosine window
disp('------ Example 2: Uniform BARKlet FB with fractional sub-sampling and cosine windows ------')
[barklets,a2,fc2,L2]=audfilters(sr,Ls,'bark','fractionaluniform','spacing',spac,'redmul',red_fac,'cosine');

s2 = postpad(s,L2);

% Compute and display FB properties and redundancy
K2 = length(a2)-1;
fprintf('The filter bank features K = %i filters from 0 to %i Hz + one at Nyquist\n',K2,round(fc2(end-1)));
fprintf('Filters'' spacing V = %i filter(s)/Bark\n',V);
fprintf('Redundancy factor = %g\n',red_fac);
R2 = (sum(2*a2(:,2))-sum(a2([1,end],2)))/L2;
fprintf('Actual redundancy of the system: %g \n', R2);

% Analysis
c_bark = filterbank(s2,barklets,a2);

% Synthesis
if R2 >= 1
    if red_fac >= 1 % Painless case, compute the dual filters directly 
        barkletsd = filterbankrealdual(barklets,a2,L2);
        s_r2 = 2*real(ifilterbank(c_bark,barkletsd,a2));
    else % Non-painless, apply the canonical dual frame iteratively using CG   
        disp('An iterative synthesis using CG is performed...')
        F  = frame('filterbankreal',barklets,a2,length(barklets));
        c_F=framenative2coef(F,c_bark);
        [s_r2,~,~] = frsyniter(F,c_F,L2,'tol',10^-14);
    end
else
    disp('Redundancy < 1, no synthesis is performed')
    s_r2 = 0;
end
    
err2 = norm(s2-s_r2)/norm(s2);
fprintf('Relative error of reconstruction: %e \n\n',err2);

%% Do figures...
if PLOT
%   Plot frequency responses of individual filters
    if red_fac >= 1
        figure('Name','Example 1: Cosine Filter responses');
        subplot(2,1,1);
        filterbankfreqz(barklets,a2,L2,sr,'plot','linabs','posfreq');
        title('Analysis filters')
        subplot(2,1,2);
        filterbankfreqz(barkletsd,a2,L2,sr,'plot','linabs','posfreq');
        title('Dual filters')
    end
%     Plot FB response
    figure('Name','Example 2: FB response')
    filterbankresponse(barklets,a2,L2,sr,'real','plot');
%     Plot ERB coefficients
    figure('Name','Example 2: Analysis coefficients')
    plotfilterbank(c_bark,a2,fc2,sr,100,'audtick');
    title('Uniform BARKlet FB using cosine filters')
end

%% 3rd Example: Uniform FB on the Mel scale with 40 filters from fmin to fmax, integer sub-sampling, and a triangular window
disp('------ Example 3: Uniform Mel FB with integer sub-sampling and triangular windows, band-limited analysis  ------')
fmin = 100;
fmax = sr/4;
[mellets,a3,fc3,L3]=audfilters(sr,Ls,fmin,fmax,'mel','uniform','M',40,'redmul',red_fac,'tria');

s3 = postpad(s,L3);

% Compute and display FB properties and redundancy
K3 = length(a3)-1;
melspac = (K3-1)/(freqtoaud(fmax,'mel')-freqtoaud(fmin,'mel'));
fprintf('The filter bank features K = %i filters from 0 to %i Hz + one at Nyquist\n',K3,fmax);
fprintf('Two additional filters are positioned at zero and Nyquist frequencies\n');
fprintf('Filters'' spacing V = %3.3f filter(s)/Mel\n',melspac);
fprintf('Redundancy factor = %g\n',red_fac);
R3 = sum(2*a3.^-1)-sum(a3([1,end],1).^-1);
fprintf('Actual redundancy of the system: %g \n', R3);

% Analysis
c_mel = filterbank(s3,mellets,a3);

% Synthesis.
if R3 >= 1
    % Painless case, compute the dual filters directly 
    melletsd = filterbankrealdual(mellets,a3,L3);
    s_r3 = 2*real(ifilterbank(c_mel,melletsd,a3));
%     else % Non-painless, apply the canonical dual frame iteratively using CG 
%         disp('An iterative synthesis using CG is performed...')
%         F  = frame('filterbankreal',mellets,a3,length(mellets));
%         c_F=framenative2coef(F,c_mel);
%         [s_r3,~,~] = frsyniter(F,c_F,L3,'tol',10^-14);
%     end
else
    disp('Redundancy < 1, no synthesis is performed')
    s_r3 = 0;
end
err3 = norm(s3-s_r3)/norm(s3);
fprintf('Relative error of reconstruction: %e \n\n',err3);

%% Do figures...
if PLOT
%   Plot frequency responses of individual filters
    if red_fac >= 1
        figure('Name','Example 3: Triangular Filter responses');
        subplot(2,1,1);
        filterbankfreqz(mellets,a3,L3,sr,'plot','linabs','posfreq');
        title('Analysis filters')
        subplot(2,1,2);
        filterbankfreqz(melletsd,a3,L3,sr,'plot','linabs','posfreq');
        title('Dual filters')
    end
%     Plot FB response
    figure('Name','Example 3: FB response')
    filterbankresponse(mellets,a3,L3,sr,'real','plot');
%     Plot ERB coefficients
    figure('Name','Example 3: Analysis coefficients')
    plotfilterbank(c_mel,a3,fc3,sr,100,'audtick');
    title('Uniform MELlet FB using 40 triangular filters')
end

%% 4th Example: Non-uniform FB on the ERB scale with fractional sub-sampling and a truncated Gaussian window
% The previous examples used FIR windows. Here we show how to construct
% Gaussian-windowed filters. Gaussians are truncated in the frequency
% domain so that supp(H_k) = 4*ERB(fc_k) in Hz. The downsampling factors 
% are then computed to satisfy a painless system.
disp('------ Example 4: Non-uniform ERBlet FB with fractional sub-sampling and truncated Gaussian windows ------')
erblets2 = cell(K1+1,1);% Cell array of analysis windows
gamma =  audfiltbw(fc1);% ERB scale
fsupp = 4*gamma;% Frequency supports of the windows in Hz

% Find suitable channel subsampling rates
aprecise=sr./fsupp/red_fac;
aprecise=aprecise(:);
N = ceil(L1./aprecise);
a4 = [repmat(L1,K1+1,1),N];
scal = sqrt(a4(:,1)./a4(:,2));% Scaling factors of the filters
% Re-scale the first and last channels
scal(1) = 1/sqrt(2)*scal(1);
scal(end) = 1/sqrt(2)*scal(end);

% Compute and display FB properties and redundancy
K4 = length(a4)-1;
fprintf('The filter bank features K = %i filters from 0 to %i Hz + one at Nyquist\n',K4,round(fc1(end-1)));
fprintf('Filters'' spacing V = %i filter(s)/ERB\n',V);
fprintf('Redundancy factor = %g\n',red_fac);
R4 = (sum(2*a4(:,2))-sum(a4([1,end],2)))/L1;
fprintf('Actual redundancy of the system: %g \n', R4);

% % Compute Gaussian windows
for k=1:K1+1
    xlh = fsupp(k)/2;
    x = linspace(-2,2,fsupp(k)*L1/sr)';
    gf = scal(k).*exp(-pi.*((.79*x).^2));    
    f_off = floor((fc1(k)-xlh)*L1/sr);
    erblets2{k}.H = gf;
    erblets2{k}.foff = f_off;
    erblets2{k}.L = L1;
    erblets2{k}.realonly = 0;
end

% Analysis
c_erb2 = filterbank(s1,erblets2,a4);

% Synthesis
if R4 >= 1
    if red_fac >= 1 % Painless case, compute the dual filters directly 
        erbletsd2 = filterbankrealdual(erblets2,a4,L1);
        s_r4 = 2*real(ifilterbank(c_erb2,erbletsd2,a4));
    else % Non-painless, apply the canonical dual frame iteratively using CG
        disp('An iterative synthesis using CG is performed...')
        F  = frame('filterbankreal',erblets2,a4,length(erblets2));
        c_F=framenative2coef(F,c_erb2);
        [s_r4,~,~] = frsyniter(F,c_F,L1,'tol',10^-14);%,'maxit',1000);
    end
    err4 = norm(s1-s_r4)/norm(s1);
    fprintf('Relative error of reconstruction: %e \n\n',err4);
else 
    disp('Redundancy < 1, no synthesis is performed')
end

%% Do figures...
if PLOT
%     Plot frequency responses of individual filters
    if red_fac >= 1
        figure('Name','Example 4: Gaussian filter responses');
        subplot(2,1,1);
        filterbankfreqz(erblets2,a4,Ls,sr,'plot','linabs','posfreq');
        title('Analysis filters')
        subplot(2,1,2);
        filterbankfreqz(erbletsd2,a4,Ls,sr,'plot','linabs','posfreq');
        title('Dual filters')
    end
%     Plot FB response
    figure('Name','Example 4: FB response')
    filterbankresponse(erblets2,a4,Ls,sr,'real','plot');
%     Plot ERB coefficients
    figure('Name','Example 4: Analysis coefficients')
    plotfilterbank(c_erb2,a4,fc1,sr,100,'audtick');
    title('Non-uniform ERBlet FB using Gaussian filters')
end

%% 5th Example: Non-uniform FB on the ERB scale with fractional sub-sampling and periodized Gaussian windows
disp('------ Example 5: Non-uniform FB with fractional sub-sampling and periodized Gaussian windows ------')
erblets3 = cell(K1+1,1);% Cell array of analysis windows
gamma =  audfiltbw(fc1);% ERB scale
fsupp = 4*gamma;% Frequency supports of the windows in Hz

% Find suitable channel subsampling rates
aprecise=sr./fsupp/red_fac;
aprecise=aprecise(:);
N = ceil(L1./aprecise);
a5 = [repmat(L1,K1+1,1),N];
scal = sqrt(a5(:,1)./a5(:,2));% Scaling factors of the filters
% Re-scale the first and last channels
scal(1) = 1/sqrt(2)*scal(1);
scal(end) = 1/sqrt(2)*scal(end);

% Compute and display FB properties and redundancy
K5 = length(a5)-1;
fprintf('The filter bank features K = %i filters from 0 to %i Hz + one at Nyquist\n',K5,round(fc1(end-1)));
fprintf('Filters'' spacing V = %i filter(s)/ERB\n',V);
fprintf('Redundancy factor = %g\n',red_fac);
R5 = (sum(2*a5(:,2))-sum(a5([1,end],2)))/L1;
fprintf('Actual redundancy of the system: %g \n', R5);

% % Compute Gaussian windows
for k=1:K1+1
    xlh = fsupp(k)/2;
    gf = fftshift(pgauss(ceil(fsupp(k)*Ls/sr),'width',round((1/.79)*gamma(k)/sr*L1)));
    gf = scal(k)*normalize(gf,'inf');
    f_off = floor((fc1(k)-xlh)*L1/sr);
    %f_off = floor(fc1(k)*Ls/sr-N(k)/2);
    erblets3{k}.H = gf;
    erblets3{k}.foff = f_off;
    erblets3{k}.L = L1;
    erblets3{k}.realonly = 0;
end

% Analysis
c_erb3 = filterbank(s1,erblets3,a5);

% Synthesis
if R5 >= 1
    if red_fac >= 1 % Painless case, compute the dual filters directly 
        erbletsd3 = filterbankrealdual(erblets3,a5,L1);
        s_r5 = 2*real(ifilterbank(c_erb3,erbletsd3,a5));
    else % Non-painless, apply the canonical dual frame iteratively using CG
        disp('An iterative synthesis using CG is performed...')
        F  = frame('filterbankreal',erblets3,a5,length(erblets3));
        c_F=framenative2coef(F,c_erb3);
        [s_r5,~,~] = frsyniter(F,c_F,L1,'tol',10^-14);%,'maxit',1000);
    end
    err5 = norm(s1-s_r5)/norm(s1);
    fprintf('Relative error of reconstruction: %e \n\n',err5);
else 
    disp('Redundancy < 1, no synthesis is performed')
end

%% Do figures...
if PLOT
%     Plot frequency responses of individual filters
    if red_fac >= 1
        figure('Name','Example 5: Gaussian filter responses');
        subplot(2,1,1);
        filterbankfreqz(erblets3,a5,L1,sr,'plot','linabs','posfreq');
        title('Analysis filters')
        subplot(2,1,2);
        filterbankfreqz(erbletsd3,a5,L1,sr,'plot','linabs','posfreq');
        title('Dual filters')
    end
%     Plot FB response
    figure('Name','Example 5: FB response')
    filterbankresponse(erblets3,a5,L1,sr,'real','plot');
%     Plot ERB coefficients
    figure('Name','Example 5: Analysis coefficients')
    plotfilterbank(c_erb3,a5,fc1,sr,100,'audtick');
    title('Non-uniform ERBlet FB using Gaussian filters')
end

% eof